/******************************************************************************
* C Source: Extraction_Set                                                    
*                                                                             
* File:                                                                       
*    extrctset.c                                                              
*                                                                             
* Description:                                                                
*    This set of functions performs various manipulations on a linked list    
*    (see lnkd_lst.c and lnkd_lst.h) of special Extraction Lists              
*    (see extrtlst.c extrtlst.h), each node for an individual extraction      
*    list. (see extrtset.h). Each extraction list is assumed to consist of    
*    extractions from plates that reside on the same CD-ROM disc.             
*                                                                             
* Module List:                                                                
*    Add_Extraction_List_to_Set                                               
*       Insert an extraction list into the top of an extraction set.          
*    Create_Empty_Extraction_Set                                              
*       Create a new empty extraction set.                                    
*    Delete_Extraction_Set                                                    
*       Delete the linked list for an extraction set, but not the lists it    
*       points to.                                                            
*    Delete_Extraction_Set_Lists                                              
*       Delete the linked list for an extraction set, AND not the lists it    
*       points to including the data of the extraction lists.                 
*    Get_Lowest_Disc_List_from_Set                                            
*       Get the list in the set for the lowest disc number in the set.        
*    Insert_List_into_Extraction_Set                                          
*       Add each extraction in an input extraction list to the extraction     
*       list in the set for the disc of the extraction.                       
*                                                                             
* History:                                                                    
*    Created    : 15-JUN-93 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
*                                                                             
*    Updated    : 11-SEP-96 by J. Doggett,                                    
*       Redelivered Insert_List_into_Extraction_Set and              
*          Get_Lowest_Disc_List_from_set functions.                           
*    Redelivered: xx-xxx-xx by J. Doggett,                                    
*       GetImage Version 2.0x.                                                
*                                                                             
* Copyright (c) 1993, 1994, 1996, Association of Universities for Research in 
* Astronomy, Inc.  All Rights Reserved.                                       
* Produced under National Aeronautics and Space Administration grant          
* NAG W-2166.                                                                 
******************************************************************************/
#include <stdio.h>

#include "extrtset.h"

#include "extrctn.h"
#include "extrtlst.h"
#include "lnkd_lst.h"

/******************************************************************************
* Module:                                                                     
*    Add_Extraction_List_to_Set                                               
*                                                                             
* File:                                                                       
*    extrtset.c                                                               
*                                                                             
* Description:                                                                
*    Adds an extraction list to the top of an extraction set.                 
*                                                                             
* Return Value:                                                               
*    none                                                                     
*                                                                             
* Arguments;                                                                  
*                                                                             
*    Input Arguments:                                                         
*       X_Set                                                                 
*          X_SET, Reference                                                   
*             The extraction set to which we're adding the extraction list.   
*       X_List                                                                
*          X_LIST, Reference                                                  
*             The extraction list we're adding to the extraction set.         
*    Input/Output Arguments:                                                  
*       none                                                                  
*    Output Arguments:                                                        
*       none                                                                  
*                                                                             
* History:                                                                    
*    Created    : 15-JUN-93 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
******************************************************************************/
/******************************************
* Add_Extraction_List_to_Set declarations *
******************************************/
void   Add_Extraction_List_to_Set( X_Set, X_List )
X_SET  X_Set ;
X_LIST X_List;
{
/***************************************************
* Add_Extraction_List_to_Set begin executable code *
***************************************************/
   Insert_Linked_List_Data( X_Set, X_List );

   return;
}
/*************************************************
* Add_Extraction_List_to_Set end executable code *
*************************************************/

/******************************************************************************
* Module:                                                                     
*    Create_Empty_Extraction_Set                                              
*                                                                             
* File:                                                                       
*    extrtset.c                                                               
*                                                                             
* Description:                                                                
*    Create a new empty extraction set, allocating memory for the header.     
*                                                                             
* Return Value:                                                               
*    X_LIST: The newly created extraction.                                    
*                                                                             
* Arguments;                                                                  
*                                                                             
*    Input Arguments:                                                         
*       none                                                                  
*    Input/Output Arguments:                                                  
*       none                                                                  
*    Output Arguments:                                                        
*       none                                                                  
*                                                                             
* History:                                                                    
*    Created    : 15-JUN-93 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
******************************************************************************/
/*******************************************
* Create_Empty_Extraction_Set declarations *
*******************************************/
X_SET Create_Empty_Extraction_Set()
{
/****************************************************
* Create_Empty_Extraction_Set begin executable code *
****************************************************/
   return ( X_SET )Create_Empty_Linked_List();
}
/**************************************************
* Create_Empty_Extraction_Set end executable code *
**************************************************/

/******************************************************************************
* Module:                                                                     
*    Delete_Extraction_Set                                                    
*                                                                             
* File:                                                                       
*    extrtset.c                                                               
*                                                                             
* Description:                                                                
*    Free memory allocated for an extraction set, but not the data it points  
*    to.                                                                      
*                                                                             
* Return Value:                                                               
*    none                                                                     
*                                                                             
* Arguments;                                                                  
*                                                                             
*    Input Arguments:                                                         
*       X_Set                                                                 
*          X_SET, Reference                                                   
*             The extraction set we're deleting from memory.                  
*    Input/Output Arguments:                                                  
*       none                                                                  
*    Output Arguments:                                                        
*       none                                                                  
*                                                                             
* History:                                                                    
*    Created    : 15-JUN-93 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
******************************************************************************/
/*************************************
* Delete_Extraction_Set declarations *
*************************************/
void  Delete_Extraction_Set( X_Set )
X_SET X_Set;
{
/**********************************************
* Delete_Extraction_Set begin executable code *
**********************************************/
   Delete_Linked_List( X_Set );

   return;
}
/********************************************
* Delete_Extraction_Set end executable code *
********************************************/

/******************************************************************************
* Module:                                                                     
*    Delete_Extraction_Set_Lists                                              
*                                                                             
* File:                                                                       
*    extrctset.c                                                              
*                                                                             
* Description:                                                                
*    Free memory allocated for extraction set, but AND the extraction lists   
*    it points to including the data of the extraction lists.                 
*                                                                             
* Return Value:                                                               
*    none                                                                     
*                                                                             
* Arguments;                                                                  
*                                                                             
*    Input Arguments:                                                         
*       X_Set                                                                 
*          EXTRACTION_SET, Reference                                          
*             The extraction set we're deleting.                              
*    Input/Output Arguments:                                                  
*       none                                                                  
*    Output Arguments:                                                        
*       none                                                                  
*                                                                             
* History:                                                                    
*    Created    : 17-AUG-93 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
******************************************************************************/
/*******************************************
* Delete_Extraction_Set_Lists declarations *
*******************************************/
void  Delete_Extraction_Set_Lists( X_Set )
X_SET X_Set;
{
/****************************************************
* Delete_Extraction_Set_Lists begin executable code *
****************************************************/
   Delete_Linked_List_Data( X_Set, Delete_Extraction_List_Data );

   return;
}
/**************************************************
* Delete_Extraction_Set_Lists end executable code *
**************************************************/

/******************************************************************************
* Module:                                                                     
*    Get_Lowest_Disc_List_from_Set                                            
*                                                                             
* File:                                                                       
*    extrctset.c                                                              
*                                                                             
* Description:                                                                
*    Get the list in an extraction set with volume label greater than that of 
*    the input basement extraction list.                                      
*                                                                             
* Return Value:                                                               
*    X_LIST: Extraction list found. NULL is returned if no list has disc      
*    greater than or equal to the input disc number.                          
*                                                                             
* Arguments;                                                                  
*                                                                             
*    Input Arguments:                                                         
*       X_set                                                                 
*          X_SET, Reference                                                   
*             Extraction set we're searching.                                 
*       Basement_Extraction_List                                              
*          X_LIST, Reference                                                  
*             The extraction list returned will have volume label greater     
*             than Basement_Extraction_List                                   
*    Input/Output Arguments:                                                  
*       none                                                                  
*    Output Arguments:                                                        
*       none                                                                  
*                                                                             
* History:                                                                    
*    Created    : 23-AUG-93 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
*                                                                             
*    Updated    : 11-SEP-96 by J. Doggett,                                    
*       Modified to compare volume labels rather than disc numbers.           
*    Redelivered: 11-SEP-96 by J. Doggett,                                    
*       GetImage Version 2.0x.                                                
******************************************************************************/
/*********************************************
* Get_Lowest_Disc_List_from_Set declarations *
*********************************************/
X_LIST Get_Lowest_Disc_List_from_Set( X_Set, Basement_Extraction_List)
X_SET  X_Set                   ;
X_LIST Basement_Extraction_List;
{
   XL_NODE Current_Disc_List_Node  ; /* Node of current list we're checking */
   X_LIST  Current_Disc_List       ; /* Current list we're checking         */
   X_LIST  Lowest_Disc_List        ;

/******************************************************
* Get_Lowest_Disc_List_from_Set begin executable code *
******************************************************/
   Lowest_Disc_List = (X_LIST)NULL;

   if (X_Set == (X_SET)NULL)
      return Lowest_Disc_List;

   Current_Disc_List_Node = (XL_NODE)(X_Set->Top);

/* *** Get_Lowest_Disc_List_from_Set ***                                   2 */


   if (Current_Disc_List_Node != (XL_NODE)NULL)
      Current_Disc_List = (X_LIST )(Current_Disc_List_Node->Data);

   while ( Current_Disc_List_Node != NULL )
   {
      if ((Basement_Extraction_List == (X_LIST)NULL  ) ||
          (Compare_Disc_Lists_VolumeLabels
                 (Current_Disc_List,
		  Basement_Extraction_List)      > 0)  ) 
      {
         if (Lowest_Disc_List == (X_LIST)NULL)
            Lowest_Disc_List = Current_Disc_List;
         else if (
            Compare_Disc_Lists_VolumeLabels
	          (Current_Disc_List, Lowest_Disc_List) < 0)
            Lowest_Disc_List = Current_Disc_List;
      }

      Current_Disc_List_Node = Current_Disc_List_Node->Next;
      if (Current_Disc_List_Node != (XL_NODE)NULL)
         Current_Disc_List = ( X_LIST )( Current_Disc_List_Node->Data );
   }
   return Lowest_Disc_List;
 }
/****************************************************
* Get_Lowest_Disc_List_from_Set end executable code *
****************************************************/

/******************************************************************************
* Module:                                                                     
*    Insert_List_into_Extraction_Set                                          
*                                                                             
* File:                                                                       
*    extrtset.c                                                               
*                                                                             
* Description:                                                                
*    Takes each extraction from an extration list and inserts into the list   
*    in an extraction set with the same corresponding CD-ROM disc.            
*                                                                             
* Return Value:                                                               
*    none                                                                     
*                                                                             
* Arguments;                                                                  
*                                                                             
*    Input Arguments:                                                         
*       X_Set                                                                 
*          X_SET, Reference                                                   
*             The extraction set to which we're adding the extraction list.   
*       X_List                                                                
*          X_LIST, Reference                                                  
*             The extraction list we're adding to the extraction set.         
*    Input/Output Arguments:                                                  
*       none                                                                  
*    Output Arguments:                                                        
*       none                                                                  
*                                                                             
* History:                                                                    
*    Created    : 15-JUN-93 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
*                                                                             
*    Updated    : 11-SEP-96 by J. Doggett,                                    
*       Modified to use volume label for finding list in extraction set.      
*    Redelivered: 11-SEP-96 by J. Doggett,                                    
*       GetImage Version 2.0x.                                                
******************************************************************************/
/***********************************************
* Insert_List_into_Extraction_Set declarations *
***********************************************/
void     Insert_List_into_Extraction_Set( X_Set, Input_X_List )
X_SET    X_Set       ;
X_LIST   Input_X_List;
{
   XL_NODE    Input_XL_Node   ; /* Node of an extraction in the input list   */
   EXTRACTION Input_Extraction; /* Extraction from the input list            */
   X_LIST     Disc_X_List     ; /* Extraction list for a given disc from the *
                                 *    input set.                             */

/********************************************************
* Insert_List_into_Extraction_Set begin executable code *
********************************************************/
   Input_XL_Node = Input_X_List->Top;

/*
 * Loop through extractions of the input extraction list, and put them into
 * the appropriate list of input extraction set according to CD-ROM disc
 * number.
 */

/* *** Insert_List_into_Extraction_Set ***                                 2 */


   while ( Input_XL_Node != NULL )
   {
      Input_Extraction = ( EXTRACTION )( Input_XL_Node->Data );
/*
 * Find appropriate list in the extraction set.
 */
      Disc_X_List =
         Find_Linked_List_Data(
                                 X_Set                               ,
                                 Input_Extraction->Plate->volumeLabel,
                                 Check_Disc_Extraction_List_Disc
                              );

/*
 * If we couldn't find a list for the disc, then create one.
 */
      if ( Disc_X_List == NULL )
      {
         Disc_X_List = Create_Empty_Extraction_List();
         Add_Extraction_List_to_Set( X_Set, Disc_X_List );
      }

/*
 * Put the extraction into the list in the extraction set
 */
      Add_Extraction_to_List( Disc_X_List, Input_Extraction  );
      Input_XL_Node = Input_XL_Node->Next;
   }
   return;
}
/******************************************************
* Insert_List_into_Extraction_Set end executable code *
******************************************************/
